#include <fstream.h>
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <math.h>
#include <string.h>

#define TEX_SIZE    12288
#define BMP_OFFS    0x36
#define TEX_WIDTH   64
#define TEX_HEIGHT  64
#define R           0.7
#define H           5.0
#define STEP        M_PI * 0.05

const GLfloat K = 360.0 / 20.0 * M_PI / 180.0;
const double RAD = M_PI / 180;

char **textures;
UINT timer;

void Error(char* filename)
{
  char t[100] = "   ";
  for (int i = 0; i < strlen(filename); i++)
    filename[i] = (char)toupper(filename[i]);
  strcat(t, filename);
  MessageBox(0, t, "", MB_OK + MB_ICONSTOP);
  ExitProcess(-1);
}

class Util {
    void DrawSphere(GLfloat, int, int);
    void DrawCylinder();
  public:
    void DrawShotGun();
    void DrawShot()
    {
      DrawSphere(0.04, 10, 10);
    }
};

void Util::DrawSphere(GLfloat r, int slices, int segments)
{
  GLfloat old_x = r, old_y = 0, x, y;
  GLfloat C1 = 360.0 / (GLfloat)segments, C2 = 2.0 * r / (GLfloat)slices;
  GLfloat old_z = -r, z = -r, r2, old_r = 0;
  int i, j = -slices / 2;

  for (; j < slices / 2; j++)
  {
    z += C2;
    if (r * r - z * z < 0)
      r2 = 0;
    else
      r2 = sqrt(r * r - z * z);
    for (i = 0; i < segments + 1; i++)
    {
      x = cos(i * C1 * RAD);
      y = sin(i * C1 * RAD);
      glBegin(GL_QUADS);
        glVertex3f(x * r2, y * r2, z);
        glVertex3f(old_x * r2, old_y * r2, z);
        glVertex3f(old_x * old_r, old_y * old_r, old_z);
        glVertex3f(x * old_r, y * old_r, old_z);
      glEnd();
      old_x = x;
      old_y = y;
    }
    old_z = z;
    old_r = r2;
  }
}

void Util::DrawCylinder()
{
  GLfloat x, y;
  int i = 1;

  glBegin(GL_QUAD_STRIP);
    glNormal3f(1.0, 0.0, 0.0);
    glVertex3f(R, 0.0, -H);
    glVertex3f(R, 0.0, H);
    for (; i <= 10; i++)
    {
      x = cos(i * K) * R;
      y = sin(i * K) * R;
      glNormal3f(x / R, 0, y / R);
      glVertex3f(x, y, -H);
      glVertex3f(x, y, H * 4);
    }
  glEnd();
}

void Util::DrawShotGun()
{
  GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
  GLfloat local_view[] = { 0.0 };
  GLfloat no_mat [] = { 0.0, 0.0, 0.0 };
  GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat mat_ambient[] = { 0.3, 0.3, 0.3, 1.0 };
  GLfloat high_shininess[] = { 50.0 };

  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glTranslatef(-0.7, -3.0, -16);
  glRotatef(7, 1.0, 0.0, 0.0);

  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, no_mat);

  DrawCylinder();
  glTranslatef(1.4, 0.0, 0.0);
  DrawCylinder();

  glDisable(GL_LIGHTING);
}

class LevelMap {
  public:
    char* Map;
    long* Pointers;
    short* Sizes;
    char* Polygons;
    short* VisList;
    char* StartPos;
    LevelMap(char *);
    ~LevelMap()
    {
      delete StartPos;
      delete VisList;
      delete Polygons;
      delete Sizes;
      delete Pointers;
      delete Map;
    }
} *map;

LevelMap::LevelMap(char* filename)
{
  short N;
  long M;
  int pos = 0;

  ifstream in(filename, ios::binary);
  if (!in) Error(filename);

  Map = new char[4096];
  Pointers = new long[4096];
  Sizes = new short[4096];

  in.read(Map, 4096);
  in.read((char *)Pointers, 16384);
  in.read((char *)Sizes, 8192);

  in.read((char *)&N, sizeof(short));
  Polygons = new char[N * 4];
  in.read(Polygons, N * 4);

  in.read((char *)&M, sizeof(long));
  VisList = new short[M];
  in.read((char *)VisList, M * 2);

  for (int i = 0; i < 4096; i++)
    if (Map[i] == 2)
      pos++;

  StartPos = new char[pos * 2];
  pos = 0;
  for (int i = 0; i < 4096; i++)
    if (Map[i] == 2)
    {
      StartPos[pos] = char(i / 64);
      StartPos[pos + 1] = char(i % 64);
      pos += 2;
    }
}

class Player {
  public:
    double org_z;
    double x, y, z;
    double shot_time;
    double y_angle, z_angle, x_angle;
    bool jumping;
    double jump_time;
    Player(double x1, double y1, double z1, double angle1)
    {
      SetPos(x1, y1);
      org_z = z1;
      SetZ(0);
      x_angle = 0;
      y_angle = angle1;
      z_angle = 0;
      shot_time = M_PI * 2;
      jumping = false;
    }
    void SetPos(double x1, double y1)
    {
      x = x1;
      y = y1;
    }
    void MoveX(double x1)
    {
      x += x1;
    }
    void MoveY(double y1)
    {
      y += y1;
    }
    void SetZ(double z1)
    {
      z = z1 + org_z;
    }
    void RotateY(double a)
    {
      y_angle += a;

      if (x_angle > 360.0)
        x_angle -= 360.0;

      if (x_angle < -360.0)
        x_angle += 360.0;
    }
    void RotateX(double a)
    {
      x_angle += a;

      if (x_angle > 90.0)
        x_angle = 90.0;

      if (x_angle < -90.0)
        x_angle = -90.0;
    }
    void HandleEvent()
    {
      if (jumping)
      {
        SetZ(sin(jump_time) * 0.9);
        jump_time += STEP;
        if (jump_time > M_PI)
          jumping = false;
      }
    }
    void Jump()
    {
      if (jumping) return;
      jumping = true;
      jump_time = 0;
    }
};

class Shot {
    double x[16], y[16], z[16], a_x[16], a_y[16];
    bool e[16];
    LevelMap* map;
  public:
    Shot(LevelMap *);
    void Fire(double, double, double, double, double);
    void Move();
    void Draw();
};

Shot::Shot(LevelMap* m)
{
  map = m;

  for (int i = 0; i < 16; i++)
    e[i] = false;
}

void Shot::Fire(double px, double py, double pz, double x_a, double y_a)
{
  for (int i = 0; i < 16; i++)
    if (!e[i])
    {
      x[i] = px;
      y[i] = py;
      z[i] = pz;
      a_x[i] = x_a;
      a_y[i] = y_a;
      e[i] = true;
      return;
    }
}

void Shot::Move()
{
  for (int i = 0; i < 16; i++)
  {
    if (e[i])
    {
      double t = cos(a_x[i] * RAD);

      x[i] += cos((a_y[i] - 90) * RAD) * 0.8 * t;
      y[i] += sin((a_y[i] - 90) * RAD) * 0.8 * t;
      z[i] += -sin(a_x[i] * RAD) * 0.8;

      if (x[i] < 0 || x[i] > 63 || y[i] < 0 || y[i] > 63 || z[i] < 0 || z[i] > 1.5)
        e[i] = false;
      else
        if (map -> Map[(int)x[i] * 64 + (int)y[i]] == 1)
          e[i] = false;
    }
  }
}

void Shot::Draw()
{
  Util u;

  for (int i = 0; i < 16; i++)
    if (e[i])
    {
      glPushMatrix();
      glTranslated(x[i], z[i], y[i]);
      u.DrawShot();
      glPopMatrix();
    }
}

class Refresh {
    LevelMap* map;
    Shot *s1, *s2;
  public:
    Refresh(LevelMap* m, Shot* s11, Shot* s22)
    {
      map = m;
      s1 = s11;
      s2 = s22;
    }
    void RefreshScreen(Player *, HDC);
};

void Refresh::RefreshScreen(Player* p, HDC hDC)
{
  Util u;
  int offs1 = (int)p -> x * 64 + (int)p -> y;
  int offs2 = map -> Pointers[offs1] / 2;
  int offs3;
  byte x1, y1, x2, y2;
  static GLfloat offs = 0.0;

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glClear(GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);

  glRotated(p -> z_angle, 0.0, 0.0, 1.0);
  glRotated(p -> x_angle, 1.0, 0.0, 0.0);
  glRotated(p -> y_angle, 0.0, 1.0, 0.0);
  glTranslated(-p -> x, -p -> z, -p -> y);

  glEnable(GL_TEXTURE_2D);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, GL_RGB, GL_UNSIGNED_BYTE, textures[2]);

  glBegin(GL_QUADS);
    glTexCoord2f(offs, offs);
    glVertex3f(0.0f, 1.5f, 0.0f);
    glTexCoord2f(offs, offs + 16.0);
    glVertex3f(0.0f, 1.5f, 64.0f);
    glTexCoord2f(offs + 16.0, offs + 16.0);
    glVertex3f(64.0f, 1.5f, 64.0f);
    glTexCoord2f(offs + 16.0, offs + 0.0);
    glVertex3f(64.0f, 1.5f, 0.0f);
  glEnd();

  glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, GL_RGB, GL_UNSIGNED_BYTE, textures[1]);

  glBegin(GL_QUADS);
    glTexCoord1i(0);
    glVertex3f(0.0f, 0.0f, 0.0f);
    glTexCoord2i(0, 64);
    glVertex3f(0.0f, 0.0f, 64.0f);
    glTexCoord2i(64, 64);
    glVertex3f(64.0f, 0.0f, 64.0f);
    glTexCoord2i(64, 0);
    glVertex3f(64.0f, 0.0f, 0.0f);
  glEnd();

  glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, GL_RGB, GL_UNSIGNED_BYTE, textures[0]);

  for (int i = 0; i < map -> Sizes[offs1]; i++)
  {
    offs3 = map -> VisList[offs2];

    x1 = map -> Polygons[offs3 * 4];
    y1 = map -> Polygons[offs3 * 4 + 1];
    x2 = map -> Polygons[offs3 * 4 + 2];
    y2 = map -> Polygons[offs3 * 4 + 3];

    if (x2 == x1)
    {
      glBegin(GL_QUADS);
        glTexCoord2i(0, 1);
        glVertex3f(x1, 1.5f, y1);
        glTexCoord2i(y2 - y1, 1);
        glVertex3f(x2, 1.5f, y2);
        glTexCoord2i(y2 - y1, 0);
        glVertex3i(x2, 0, y2);
        glTexCoord1i(0);
        glVertex3i(x1, 0, y1);
      glEnd();
    }
    else
    {
      glBegin(GL_QUADS);
        glTexCoord2i(0, 1);
        glVertex3f(x1, 1.5f, y1);
        glTexCoord2i(x2 - x1, 1);
        glVertex3f(x2, 1.5f, y2);
        glTexCoord2i(x2 - x1, 0);
        glVertex3i(x2, 0, y2);
        glTexCoord1i(0);
        glVertex3i(x1, 0, y1);
      glEnd();
    }

    offs2++;
  }

  glDisable(GL_TEXTURE_2D);
  glColor3d(0.0, 0.0, 0.2);
  s1 -> Draw();
  s2 -> Draw();

  glEnable(GL_TEXTURE_2D);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, GL_RGB, GL_UNSIGNED_BYTE, textures[2]);
  glEnable(GL_BLEND);
  glBegin(GL_QUADS);
    glColor4f(0.5, 1.0, 0.0, 0.5);
    glTexCoord2f(offs, offs);
    glVertex3f(0.0f, 0.2f, 0.0f);
    glTexCoord2f(offs, offs + 16.0);
    glVertex3f(0.0f, 0.2f, 64.0f);
    glTexCoord2f(offs + 16.0, offs + 16.0);
    glVertex3f(64.0f, 0.2f, 64.0f);
    glTexCoord2f(offs + 16.0, offs + 0.0);
    glVertex3f(64.0f, 0.2f, 0.0f);
  glEnd();
  glDisable(GL_BLEND);
  glDisable(GL_TEXTURE_2D);
  glColor4d(1.0, 1.0, 1.0, 1.0);

  glDisable(GL_DEPTH_TEST);
  glLoadIdentity();
  glRotated(p -> z_angle, 0.0, 0.0, 1.0);
  if (!p -> jumping)
    glTranslated(0.0, (p -> z - p -> org_z) * 4, 0.0);
  if (p -> shot_time < M_PI)
    glTranslated(0.0, 0.0, sin(p -> shot_time) * 8);
  u.DrawShotGun();

  offs += 0.015625;
  if (offs >= 1.0)
    offs -= 1.0;

  glFinish();
  SwapBuffers(hDC);
}

class TheGame {
    Refresh* ref;
    Player* p;
    bool u, d, l, r, sr, sl, la, ld, s;
    bool walk_mode;
    double walk_pos;
    Shot *s1, *s2;
    LevelMap* map;
    void Fire()
    {
      if (p -> shot_time < M_PI * 2) return;

      double x = cos(p -> y_angle * RAD) * 0.05;
      double y = sin(p -> y_angle * RAD) * 0.05;

      s1 -> Fire(p -> x + x, p -> y + y, p -> z, p -> x_angle, p -> y_angle);
      s2 -> Fire(p -> x - x, p -> y - y, p -> z, p -> x_angle, p -> y_angle);

      p -> shot_time = 0.0;
    }
    void TryMovePlayer(double dx, double dy)
    {
      int x = (int)p -> x;
      int y = (int)p -> y;
      double x_ = x;
      double y_ = y;
      double dx_ = dx, dy_ = dy;

      char b1 = map -> Map[(x + 1) * 64 + y];
      char b2 = map -> Map[(x - 1) * 64 + y];
      char b3 = map -> Map[x * 64 + y + 1];
      char b4 = map -> Map[x * 64 + y - 1];
      char b5 = map -> Map[(x + 1) * 64 + y - 1];
      char b6 = map -> Map[(x - 1) * 64 + y - 1];
      char b7 = map -> Map[(x - 1) * 64 + y + 1];
      char b8 = map -> Map[(x + 1) * 64 + y + 1];

      if (p -> x + dx > x_ + 0.6 && b1 == 1)
        dx_ = x_ + 0.6 - p -> x;
      else
      if (p -> x + dx < x_ + 0.4 && b2 == 1)
        dx_ = x_ + 0.4 - p -> x;
      if (p -> y + dy > y_ + 0.6 && b3 == 1)
        dy_ = y_ + 0.6 - p -> y;
      else
      if (p -> y + dy < y_ + 0.4 && b4 == 1)
        dy_ = y_ + 0.4 - p -> y;

      if (b5 == 1)
      {
        if (b4 != 1 && b1 != 1)
        {
          if (x_ + 1 - p -> x > p -> y - y_)
          {
            if (p -> x + dx > x_ + 0.6)
              dx_ = x_ + 0.6 - p -> x;
          }
          else
          {
            if (p -> y + dy < y_ + 0.4)
              dy_ = y_ + 0.4 - p -> y;
          }
        }
      }
      if (b6 == 1)
      {
        if (b4 != 1 && b2 != 1)
        {
          if (p -> x - x_ > p -> y - y_)
          {
            if (p -> x + dx < x_ + 0.4)
              dx_ = x_ + 0.4 - p -> x;
          }
          else
          {
            if (p -> y + dy < y_ + 0.4)
              dy_ = y_ + 0.4 - p -> y;
          }
        }
      }
      if (b7 == 1)
      {
        if (b2 != 1 && b3 != 1)
        {
          if (p -> x - x_ > y_ + 1 - p -> y)
          {
            if (p -> x + dx < x_ + 0.4)
              dx_ = x_ + 0.4 - p -> x;
          }
          else
          {
            if (p -> y + dy > y_ + 0.6)
              dy_ = y_ + 0.6 - p -> y;
          }
        }
      }
      if (b8 == 1)
      {
        if (b3 != 1 && b1 != 1)
        {
          if (x_ + 1 - p -> x > y_ + 1 - p -> y)
          {
            if (p -> x + dx > x_ + 0.6)
              dx_ = x_ + 0.6 - p -> x;
          }
          else
          {
            if (p -> y + dy > y_ + 0.6)
              dy_ = y_ + 0.6 - p -> y;
          }
        }
      }

      p -> MoveX(dx_);
      p -> MoveY(dy_);
    }
    void MovePlayer()
    {
      double dx = 0, dy = 0;
      double rate = 1.0;

      if (p -> jumping) rate = 0.5;

      if (l) p -> RotateY(-5.0);
      if (r) p -> RotateY(5.0);
      if (la) p -> RotateX(-5.0);
      if (ld) p -> RotateX(5.0);
      if (u)
      {
        dx = cos((p -> y_angle - 90) * RAD) * 0.4 * rate;
        dy = sin((p -> y_angle - 90) * RAD) * 0.4 * rate;
      }
      if (d)
      {
        dx = -cos((p -> y_angle - 90) * RAD) * 0.4 * rate;
        dy = -sin((p -> y_angle - 90) * RAD) * 0.4 * rate;
      }
      if (sr)
      {
        dx += cos(p -> y_angle * RAD) * 0.2 * rate;
        dy += sin(p -> y_angle * RAD) * 0.2 * rate;
      }
      if (sl)
      {
        dx -= cos(p -> y_angle * RAD) * 0.2 * rate;
        dy -= sin(p -> y_angle * RAD) * 0.2 * rate;
      }

      if (dx != 0 || dy != 0)
      {
        TryMovePlayer(dx, dy);
        if (!p -> jumping)
          p -> SetZ(sin(walk_pos) * 0.1);
        walk_pos += 0.6;

        if (walk_pos > M_PI * 2)
          walk_pos -=  M_PI * 2;
      }
    }
  public:
    TheGame(LevelMap* m)
    {
      u = d = l = r = sr = sl = la = ld = s = false;
      walk_mode = false;
      map = m;
      p = new Player(m -> StartPos[0] + 0.5, m -> StartPos[1] + 0.5, 0.5, 120.0);
      s1 = new Shot(m);
      s2 = new Shot(m);
      ref = new Refresh(m, s1, s2);
    }
    void paint(HDC hDC)
    {
      MovePlayer();
      p -> HandleEvent();

      if (s) Fire();

      if (p -> shot_time < M_PI * 2)
        p -> shot_time += 1.0;
      else
        p -> shot_time = M_PI * 2;

      s1 -> Move();
      s2 -> Move();

      ref -> RefreshScreen(p, hDC);
    }
    void keyDown(int);
    void keyUp(int);
} *game;

void TheGame::keyDown(int key)
{
  switch (key)
  {
    case 38: u = true; d = false; break;
    case 40: d = true; u = false; break;
    case 37: l = true; r = false; break;
    case 39: r = true; l = false; break;
    case 36: p -> x_angle = 0.0; break;
    case 188: sl = true; sr = false; p -> z_angle = -3.0; break;
    case 190: sr = true; sl = false; p -> z_angle = 3.0; break;
    case 65:
    case 97: la = true; ld = false; break;
    case 90:
    case 122: ld = true; la = false; break;
    case 32: s = true; break;
    case 'q':
    case 'Q': p -> Jump();
  }

  if (u || d || sl || sr)
  {
    if (!walk_mode)
      walk_pos = 0.0;
    walk_mode = true;
  }
}

void TheGame::keyUp(int key)
{
  switch (key)
  {
    case 38: u = false; break;
    case 40: d = false; break;
    case 37: l = false; break;
    case 39: r = false; break;
    case 188: sl = false; p -> z_angle = 0.0; break;
    case 190: sr = false; p -> z_angle = 0.0;
    case 65:
    case 97: la = false; break;
    case 90:
    case 122: ld = false; break;
    case 32: s = false;
  }

  if (!u && !d && !sl && !sr)
  {
    p -> SetZ(0.0);
    walk_mode = false;
  }
}

void setDCPixelFormat(HDC hDC)
{
  static PIXELFORMATDESCRIPTOR pfd =
  {
    sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
    1,                                      // version number
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
    | PFD_DOUBLEBUFFER,                     // flags
    PFD_TYPE_RGBA,                          // RGBA pixel values
    16,                                     // 24-bit color
    0, 0, 0, 0, 0, 0,                       // don't care about these
    0, 0,                                   // no alpha buffer
    0, 0, 0, 0, 0,                          // no accumulation buffer
    16,                                     // 32-bit depth buffer
    0,                                      // no stencil buffer
    0,                                      // no auxiliary buffers
    PFD_MAIN_PLANE,                         // layer type
    0,                                      // reserved(must be 0)
    0, 0, 0                                 // no layer masks
  };
  int pixelFormat;

  pixelFormat = ChoosePixelFormat(hDC, &pfd);
  SetPixelFormat(hDC, pixelFormat, &pfd);
  DescribePixelFormat(hDC, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
}

void InitializeGL()
{
  glDepthFunc(GL_LEQUAL);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, 1);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, 1);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

LONG WINAPI wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HDC hDC;
  static HGLRC hrc;
  PAINTSTRUCT ps;
  GLdouble aspect;
  GLsizei width, height;

  switch(msg)
  {
    case WM_KEYDOWN:
      if (wParam == 27)
        DestroyWindow(hWnd);
      else
        game -> keyDown(wParam);
      return 0;

    case WM_KEYUP:
      game -> keyUp(wParam);
      return 0;

    case WM_CREATE:
      hDC = GetDC(hWnd);
      setDCPixelFormat(hDC);
      hrc = wglCreateContext(hDC);
      wglMakeCurrent(hDC, hrc);
      InitializeGL();
      timer = SetTimer(hWnd, 1, 50, NULL);
      return 0;

    case WM_SIZE:
      width = (GLsizei)LOWORD(lParam);
      height = (GLsizei)HIWORD(lParam);
      if (!width || !height) return 0;

      aspect = (GLdouble)width /(GLdouble)height;
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(30.0, aspect, 0.1, 64.0);
      glViewport(0, 0, width, height);

      return 0;

    case WM_PAINT:
      BeginPaint(hWnd, &ps);
      game -> paint(hDC);
      EndPaint(hWnd, &ps);
      return 0;

    case WM_DESTROY:
      wglMakeCurrent(NULL, NULL);
      wglDeleteContext(hrc);
      KillTimer(hWnd, timer);
      PostQuitMessage(0);
      return 0;

    case WM_TIMER:
      InvalidateRect(hWnd, NULL, FALSE);
      return 0;
  }

  return DefWindowProc(hWnd, msg, wParam, lParam);
}

void LoadTextures()
{
  int i = 0;
  char filename[] = "texture_.bmp";
  ifstream in;

  textures = new char *[3];

  for (; i < 3; i++)
  {
    textures[i] = new char[TEX_SIZE];
    filename[7] = char(i + '1');

    in.open(filename, ios::binary);
    if (!in) Error(filename);

    in.seekg(BMP_OFFS, ios::beg);
    in.read(textures[i], TEX_SIZE);
    in.close();
  }
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdLine, int cmdShow)
{
  static char appName [] = "World - by HackMaster";
  WNDCLASS wc;
  HWND hWnd;
  MSG msg;

  LoadTextures();
  map = new LevelMap("level.map");
  game = new TheGame(map);

  wc.style = 0;
  wc.lpfnWndProc =(WNDPROC)wndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground =(HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName = (LPCTSTR)0;
  wc.lpszClassName = appName;

  RegisterClass(&wc);

  hWnd = CreateWindow(appName, appName,
                      WS_OVERLAPPEDWINDOW,
                      CW_USEDEFAULT, CW_USEDEFAULT, 320,
                      240, HWND_DESKTOP, NULL, hInstance, NULL);

  ShowWindow(hWnd, cmdShow);
  UpdateWindow(hWnd);

  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  delete game;
  delete map;
  for (int i = 2; i >= 0; i--)
    delete textures[i];
  delete textures;

  return msg.wParam;
}
